//------------------------------------------------------------------------------
// mastermanager.cpp
//------------------------------------------------------------------------------
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301, USA.
//
//------------------------------------------------------------------------------
// Copyright (C) 2009 "Blzut3" <admin@maniacsvault.net>
//------------------------------------------------------------------------------
#include <cassert>

#include "mastermanager.h"

#include "serverapi/masterclientsignalproxy.h"
#include "serverapi/message.h"
#include "serverapi/server.h"
#include "customservers.h"

// TODO: I don't think that MasterManager should store a duplicate of each
// server (~Zalewa).

MasterManager::MasterManager() : MasterClient()
{
	customServers = new CustomServers();
}

MasterManager::~MasterManager()
{
	clearServers();

	for (int i = 0; i < mastersReceivers.size(); ++i)
	{
		delete mastersReceivers[i];
	}

	for(int i = 0;i < masters.size();i++)
		delete masters[i];

	delete customServers;
}

void MasterManager::addMaster(MasterClient *master)
{
	if(master == NULL)
		return;

	masters.append(master);
	master->setEnabled(true);

	MasterClientSignalProxy* pMasterReceiver = new MasterClientSignalProxy(master);
	connect(pMasterReceiver, SIGNAL( listUpdated(MasterClient*) ),
		this, SLOT( masterListUpdated(MasterClient*) ) );
	connect(pMasterReceiver, SIGNAL( message(MasterClient*, const QString&, const QString&, bool) ),
		this, SIGNAL( masterMessage(MasterClient*, const QString&, const QString&, bool) ) );
	connect(pMasterReceiver, SIGNAL( messageImportant(MasterClient*, const Message&) ),
		this, SIGNAL( masterMessageImportant(MasterClient*, const Message&) ));

	mastersReceivers.append(pMasterReceiver);
}

void MasterManager::masterListUpdated(MasterClient* pSender)
{
	foreach(ServerPtr pServer, pSender->servers())
	{
		registerNewServer(pServer);
	}

	emit listUpdatedForMaster(pSender);
	mastersBeingRefreshed.remove(pSender);
	if (mastersBeingRefreshed.isEmpty())
	{
		emit listUpdated();
	}
}

MasterManager::Response MasterManager::readMasterResponse(const QByteArray &data)
{
	assert(0 && "MasterManager::readMasterResponse should not get called.");
	return RESPONSE_BAD;
}

void MasterManager::refreshStarts()
{
	setTimeouted(false);

	clearServers();

	for(int i = 0;i < masters.size();i++)
	{
		if(!masters[i]->isEnabled())
		{
			continue;
		}

		mastersBeingRefreshed.insert(masters[i]);
		masters[i]->refreshStarts();
	}
}

void MasterManager::timeoutRefreshEx()
{
	foreach(MasterClient* pMaster, mastersBeingRefreshed)
	{
		pMaster->timeoutRefresh();
	}

	mastersBeingRefreshed.clear();
}
